@@ -1,4 +1,5 @@
 /*
+ * Copyright 2010 NVIDIA Corporation
  * Copyright 1996-1997  David J. McKay
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -77,11 +78,22 @@ static Bool	NVModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode);
 static Bool	NVSetModeVBE(ScrnInfoPtr pScrn, DisplayModePtr pMode);
 
 #if XSERVER_LIBPCIACCESS
-/* For now, just match any NVIDIA PCI device and sort through them in the probe
- * routine */
+/* For now, just match any NVIDIA display device and sort through them in the
+ * probe routine */
+
+/*
+ * libpciaccess's masks are shifted by 8 bits compared to the ones in xf86Pci.h.
+ */
+#define LIBPCIACCESS_CLASS_SHIFT (PCI_CLASS_SHIFT - 8)
+#define LIBPCIACCESS_CLASS_MASK (PCI_CLASS_MASK >> 8)
+
 static const struct pci_id_match NVPciIdMatchList[] = {
-    { PCI_VENDOR_NVIDIA, PCI_MATCH_ANY, PCI_MATCH_ANY, PCI_MATCH_ANY, 0, 0, 0 },
-    { PCI_VENDOR_NVIDIA_SGS, PCI_MATCH_ANY, PCI_MATCH_ANY, PCI_MATCH_ANY, 0, 0, 0},
+    { PCI_VENDOR_NVIDIA, PCI_MATCH_ANY, PCI_MATCH_ANY, PCI_MATCH_ANY,
+      PCI_CLASS_DISPLAY << LIBPCIACCESS_CLASS_SHIFT, LIBPCIACCESS_CLASS_MASK, 0 },
+
+    { PCI_VENDOR_NVIDIA_SGS, PCI_MATCH_ANY, PCI_MATCH_ANY, PCI_MATCH_ANY,
+      PCI_CLASS_DISPLAY << LIBPCIACCESS_CLASS_SHIFT, LIBPCIACCESS_CLASS_MASK, 0 },
+
     { 0, 0, 0 }
 };
 #endif
@@ -113,7 +125,7 @@ _X_EXPORT DriverRec NV = {
 #endif
 };
 
-/* Known cards as of 2008/06/16 */
+/* Known cards as of 2010/03/08 */
 
 static SymTabRec NVKnownChipsets[] =
 {
@@ -277,6 +289,7 @@ static SymTabRec NVKnownChipsets[] =
   { 0x10DE0147, "GeForce 6700 XL" },
   { 0x10DE0148, "GeForce Go 6600" },
   { 0x10DE0149, "GeForce Go 6600 GT" },
+  { 0x10DE014A, "Quadro NVS 440" },
   { 0x10DE014C, "Quadro FX 550" },
   { 0x10DE014D, "Quadro FX 550" },
   { 0x10DE014E, "Quadro FX 540" },
@@ -292,6 +305,7 @@ static SymTabRec NVKnownChipsets[] =
   { 0x10DE0167, "GeForce Go 6200" },
   { 0x10DE0168, "GeForce Go 6400" },
   { 0x10DE0169, "GeForce 6250" },
+  { 0x10DE016A, "GeForce 7100 GS" },
 
   { 0x10DE0211, "GeForce 6800" },
   { 0x10DE0212, "GeForce 6800 LE" },
@@ -310,6 +324,7 @@ static SymTabRec NVKnownChipsets[] =
   { 0x10DE0099, "GeForce Go 7800 GTX" },
   { 0x10DE009D, "Quadro FX 4500" },
 
+  { 0x10DE01D0, "GeForce 7350 LE" },
   { 0x10DE01D1, "GeForce 7300 LE" },
   { 0x10DE01D3, "GeForce 7300 SE" },
   { 0x10DE01D6, "GeForce Go 7200" },
@@ -323,6 +338,7 @@ static SymTabRec NVKnownChipsets[] =
   { 0x10DE01DE, "Quadro FX 350" },
   { 0x10DE01DF, "GeForce 7300 GS" },
 
+  { 0x10DE0390, "GeForce 7650 GS" },
   { 0x10DE0391, "GeForce 7600 GT" },
   { 0x10DE0392, "GeForce 7600 GS" },
   { 0x10DE0393, "GeForce 7300 GT" },
@@ -339,6 +355,10 @@ static SymTabRec NVKnownChipsets[] =
   { 0x10DE0290, "GeForce 7900 GTX" },
   { 0x10DE0291, "GeForce 7900 GT" },
   { 0x10DE0292, "GeForce 7900 GS" },
+  { 0x10DE0293, "GeForce 7950 GX2" },
+  { 0x10DE0294, "GeForce 7950 GX2" },
+  { 0x10DE0295, "GeForce 7950 GT"},
+  { 0x10DE0297, "GeForce Go 7950 GTX" },
   { 0x10DE0298, "GeForce Go 7900 GS" },
   { 0x10DE0299, "GeForce Go 7900 GTX" },
   { 0x10DE029A, "Quadro FX 2500M" },
@@ -352,8 +372,14 @@ static SymTabRec NVKnownChipsets[] =
   { 0x10DE0241, "GeForce 6150 LE" },
   { 0x10DE0242, "GeForce 6100" },
   { 0x10DE0244, "GeForce Go 6150" },
+  { 0x10DE0245, "Quadro NVS 210S / NVIDIA GeForce 6150LE" },
   { 0x10DE0247, "GeForce Go 6100" },
 
+  { 0x10DE03D0, "GeForce 6150SE" },
+  { 0x10DE03D1, "GeForce 6100 nForce 405" },
+  { 0x10DE03D2, "GeForce 6100 nForce 400" },
+  { 0x10DE03D5, "GeForce 6100 nForce 420" },
+
 /*************** G8x ***************/
   { 0x10DE0191, "GeForce 8800 GTX" },
   { 0x10DE0193, "GeForce 8800 GTS" },
@@ -363,6 +389,7 @@ static SymTabRec NVKnownChipsets[] =
   { 0x10DE0400, "GeForce 8600 GTS" },
   { 0x10DE0401, "GeForce 8600 GT" },
   { 0x10DE0402, "GeForce 8600 GT" },
+  { 0x10DE0403, "GeForce 8600 GS" },
   { 0x10DE0404, "GeForce 8400 GS" },
   { 0x10DE0405, "GeForce 9500M GS" },
   { 0x10DE0407, "GeForce 8600M GT" },
@@ -386,153 +413,126 @@ static SymTabRec NVKnownChipsets[] =
   { 0x10DE0429, "Quadro NVS 140M" },
   { 0x10DE042A, "Quadro NVS 130M" },
   { 0x10DE042B, "Quadro NVS 135M" },
+  { 0x10DE042C, "GeForce 9400 GT" },
   { 0x10DE042D, "Quadro FX 360M" },
   { 0x10DE042E, "GeForce 9300M G" },
   { 0x10DE042F, "Quadro NVS 290" },
+  { 0x10DE05E0, "GeForce GTX 295" },
   { 0x10DE05E1, "GeForce GTX 280" },
   { 0x10DE05E2, "GeForce GTX 260" },
+  { 0x10DE05E3, "GeForce GTX 285" },
+  { 0x10DE05E6, "GeForce GTX 275" },
+  { 0x10DE05EB, "GeForce GTX 295" },
+  { 0x10DE05F9, "Quadro CX" },
+  { 0x10DE05FD, "Quadro FX 5800" },
+  { 0x10DE05FE, "Quadro FX 4800" },
+  { 0x10DE05FF, "Quadro FX 3800" },
   { 0x10DE0600, "GeForce 8800 GTS 512" },
+  { 0x10DE0601, "GeForce 9800 GT" },
   { 0x10DE0602, "GeForce 8800 GT" },
   { 0x10DE0604, "GeForce 9800 GX2" },
+  { 0x10DE0605, "GeForce 9800 GT" },
   { 0x10DE0606, "GeForce 8800 GS" },
+  { 0x10DE0607, "GeForce GTS 240" },
+  { 0x10DE0608, "GeForce 9800M GTX" },
   { 0x10DE0609, "GeForce 8800M GTS" },
+  { 0x10DE060A, "GeForce GTX 280M" },
+  { 0x10DE060B, "GeForce 9800M GT" },
   { 0x10DE060C, "GeForce 8800M GTX" },
   { 0x10DE060D, "GeForce 8800 GS" },
   { 0x10DE0610, "GeForce 9600 GSO" },
   { 0x10DE0611, "GeForce 8800 GT" },
   { 0x10DE0612, "GeForce 9800 GTX" },
+  { 0x10DE0613, "GeForce 9800 GTX+" },
+  { 0x10DE0614, "GeForce 9800 GT" },
+  { 0x10DE0615, "GeForce GTS 250" },
+  { 0x10DE0617, "GeForce 9800M GTX" },
+  { 0x10DE0618, "GeForce GTX 260M" },
   { 0x10DE061A, "Quadro FX 3700" },
   { 0x10DE061C, "Quadro FX 3600M" },
+  { 0x10DE061D, "Quadro FX 2800M" },
+  { 0x10DE061E, "Quadro FX 3700M" },
+  { 0x10DE061F, "Quadro FX 3800M" },
   { 0x10DE0622, "GeForce 9600 GT" },
+  { 0x10DE0623, "GeForce 9600 GS" },
+  { 0x10DE0625, "GeForce 9600 GSO 512" },
+  { 0x10DE0626, "GeForce GT 130" },
+  { 0x10DE0627, "GeForce GT 140" },
+  { 0x10DE0628, "GeForce 9800M GTS" },
+  { 0x10DE062A, "GeForce 9700M GTS" },
+  { 0x10DE062B, "GeForce 9800M GS" },
+  { 0x10DE062C, "GeForce 9800M GTS" },
+  { 0x10DE0638, "Quadro FX 1800" },
+  { 0x10DE063A, "Quadro FX 2700M" },
+  { 0x10DE0640, "GeForce 9500 GT" },
+  { 0x10DE0641, "GeForce 9400 GT" },
+  { 0x10DE0643, "GeForce 9500 GT" },
+  { 0x10DE0644, "GeForce 9500 GS" },
+  { 0x10DE0646, "GeForce GT 120" },
   { 0x10DE0647, "GeForce 9600M GT" },
   { 0x10DE0648, "GeForce 9600M GS" },
   { 0x10DE0649, "GeForce 9600M GT" },
+  { 0x10DE064A, "GeForce 9700M GT" },
   { 0x10DE064B, "GeForce 9500M G" },
+  { 0x10DE064C, "GeForce 9650M GT" },
+  { 0x10DE0652, "GeForce GT 130M" },
+  { 0x10DE0656, "GeForce 9500 GT" },
+  { 0x10DE0658, "Quadro FX 380" },
+  { 0x10DE0659, "Quadro FX 580" },
+  { 0x10DE065A, "Quadro FX 1700M" },
+  { 0x10DE065C, "Quadro FX 770M" },
+  { 0x10DE06E0, "GeForce 9300 GE" },
+  { 0x10DE06E1, "GeForce 9300 GS" },
   { 0x10DE06E4, "GeForce 8400 GS" },
   { 0x10DE06E5, "GeForce 9300M GS" },
+  { 0x10DE06E6, "GeForce G100" },
   { 0x10DE06E8, "GeForce 9200M GS" },
   { 0x10DE06E9, "GeForce 9300M GS" },
+  { 0x10DE06EA, "Quadro NVS 150M" },
+  { 0x10DE06EB, "Quadro NVS 160M" },
+  { 0x10DE06EC, "GeForce G 105M" },
+  { 0x10DE06EF, "GeForce G 103M" },
+  { 0x10DE06F8, "Quadro NVS 420" },
+  { 0x10DE06F9, "Quadro FX 370 LP" },
+  { 0x10DE06FA, "Quadro NVS 450" },
+  { 0x10DE06FD, "Quadro NVS 295" },
+  { 0x10DE0847, "GeForce 9100" },
+  { 0x10DE0848, "GeForce 8300" },
+  { 0x10DE0849, "GeForce 8200" },
+  { 0x10DE084A, "nForce 730a" },
+  { 0x10DE084B, "GeForce 9200" },
+  { 0x10DE084C, "nForce 980a/780a SLI" },
+  { 0x10DE084D, "nForce 750a SLI" },
+  { 0x10DE084F, "GeForce 8100 / nForce 720a" },
+  { 0x10DE0844, "GeForce 9100M G" },
+  { 0x10DE0845, "GeForce 8200M G" },
+  { 0x10DE0861, "GeForce 9400" },
+  { 0x10DE0862, "GeForce 9400M G" },
+  { 0x10DE0863, "GeForce 9400M" },
+  { 0x10DE086C, "GeForce 9300 / nForce 730i" },
+  { 0x10DE0872, "GeForce G102M" },
+  { 0x10DE0873, "GeForce G102M" },
+  { 0x10DE087A, "GeForce 9400" },
+  { 0x10DE087D, "ION" },
+  { 0x10DE087F, "ION LE" },
+  { 0x10DE0A20, "GeForce GT 220" },
+  { 0x10DE0A23, "GeForce 210" },
+  { 0x10DE0A2A, "GeForce GT 230M" },
+  { 0x10DE0A34, "GeForce GT 240M" },
+  { 0x10DE0A60, "GeForce G210" },
+  { 0x10DE0A62, "GeForce 205" },
+  { 0x10DE0A63, "GeForce 310" },
+  { 0x10DE0A65, "GeForce 210" },
+  { 0x10DE0A66, "GeForce 310" },
+  { 0x10DE0A74, "GeForce G210M" },
+  { 0x10DE0A78, "Quadro FX 380 LP" },
+  { 0x10DE0CA3, "GeForce GT 240" },
+  { 0x10DE0CA8, "GeForce GTS 260M" },
+  { 0x10DE0CA9, "GeForce GTS 250M" },
 
   {-1, NULL}
 };
 
-
-/*
- * List of symbols from other modules that this module references.  This
- * list is used to tell the loader that it is OK for symbols here to be
- * unresolved providing that it hasn't been told that they haven't been
- * told that they are essential via a call to xf86LoaderReqSymbols() or
- * xf86LoaderReqSymLists().  The purpose is this is to avoid warnings about
- * unresolved symbols that are not required.
- */
-
-static const char *vgahwSymbols[] = {
-    "vgaHWUnmapMem",
-    "vgaHWDPMSSet",
-    "vgaHWFreeHWRec",
-    "vgaHWGetHWRec",
-    "vgaHWGetIndex",
-    "vgaHWInit",
-    "vgaHWMapMem",
-    "vgaHWProtect",
-    "vgaHWRestore",
-    "vgaHWSave",
-    "vgaHWSaveScreen",
-    NULL
-};
-
-static const char *fbSymbols[] = {
-    "fbPictureInit",
-    "fbScreenInit",
-    NULL
-};
-
-static const char *xaaSymbols[] = {
-    "XAACopyROP",
-    "XAACreateInfoRec",
-    "XAADestroyInfoRec",
-    "XAAFallbackOps",
-    "XAAInit",
-    "XAAPatternROP",
-    NULL
-};
-
-static const char *ramdacSymbols[] = {
-    "xf86CreateCursorInfoRec",
-    "xf86DestroyCursorInfoRec",
-    "xf86InitCursor",
-    NULL
-};
-
-static const char *ddcSymbols[] = {
-    "xf86PrintEDID",
-    "xf86DoEDID_DDC2",
-    "xf86SetDDCproperties",
-    NULL
-};
-
-#ifdef XFree86LOADER
-static const char *vbeSymbols[] = {
-    "VBEInit",
-    "vbeFree",
-    "vbeDoEDID",
-    NULL
-};
-
-static const char *vbeModeSymbols[] = {
-    "VBEExtendedInit",
-    "VBEGetVBEInfo",
-    "VBEGetModePool",
-    "VBEValidateModes",
-    "VBESetModeParameters",
-    "VBEGetVBEMode",
-    "VBESetVBEMode",
-    NULL
-};
-#endif
-
-static const char *i2cSymbols[] = {
-    "xf86CreateI2CBusRec",
-    "xf86I2CBusInit",
-    NULL
-};
-
-static const char *shadowSymbols[] = {
-    "ShadowFBInit",
-    NULL
-};
-
-static const char *fbdevHWSymbols[] = {
-    "fbdevHWInit",
-    "fbdevHWUseBuildinMode",
-
-    "fbdevHWGetVidmem",
-
-    /* colormap */
-    "fbdevHWLoadPaletteWeak",
-
-    /* ScrnInfo hooks */
-    "fbdevHWAdjustFrameWeak",
-    "fbdevHWEnterVT",
-    "fbdevHWLeaveVTWeak",
-    "fbdevHWModeInit",
-    "fbdevHWSave",
-    "fbdevHWSwitchModeWeak",
-    "fbdevHWValidModeWeak",
-
-    "fbdevHWMapMMIO",
-    "fbdevHWMapVidmem",
-
-    NULL
-};
-
-static const char *int10Symbols[] = {
-    "xf86FreeInt10",
-    "xf86InitInt10",
-    NULL
-};
-
-
 #ifdef XFree86LOADER
 
 static MODULESETUPPROTO(nvSetup);
@@ -642,20 +642,6 @@ nvSetup(pointer module, pointer opts, int *errmaj, int *errmin)
 #endif
         );
 
-        /*
-         * Modules that this driver always requires may be loaded here
-         * by calling LoadSubModule().
-         */
-
-        /*
-         * Tell the loader about symbols from other modules that this module
-         * might refer to.
-         */
-        LoaderRefSymLists(vgahwSymbols, xaaSymbols, fbSymbols,
-                          ramdacSymbols, shadowSymbols,
-                          i2cSymbols, ddcSymbols, vbeSymbols,
-                          fbdevHWSymbols, int10Symbols, NULL);
-
         /*
          * The return value must be non-NULL on success even though there
          * is no TearDownProc.
@@ -784,8 +770,19 @@ NVIsG80(int chipType)
         case 0x0620:
         case 0x0630:
         case 0x0640:
+        case 0x0650:
         case 0x06e0:
         case 0x06f0:
+        case 0x0840:
+        case 0x0850:
+        case 0x0860:
+        case 0x0870:
+        case 0x0a20:
+        case 0x0a30:
+        case 0x0a60:
+        case 0x0a70:
+        case 0x0ca0:
+        case 0x0cb0:
             return TRUE;
     }
 
@@ -1213,11 +1210,9 @@ Bool NVI2CInit(ScrnInfoPtr pScrn)
     char *mod = "i2c";
 
     if (xf86LoadSubModule(pScrn, mod)) {
-        xf86LoaderReqSymLists(i2cSymbols,NULL);
 
         mod = "ddc";
         if(xf86LoadSubModule(pScrn, mod)) {
-            xf86LoaderReqSymLists(ddcSymbols, NULL);
             return NVDACi2cInit(pScrn);
         } 
     }
@@ -1316,15 +1311,16 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
 
     /* Initialize the card through int10 interface if needed */
     if (xf86LoadSubModule(pScrn, "int10")) {
- 	xf86LoaderReqSymLists(int10Symbols, NULL);
 #if !defined(__alpha__) && !defined(__powerpc__)
         xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Initializing int10\n");
         pNv->pInt = xf86InitInt10(pNv->pEnt->index);
 #endif
     }
    
+#ifndef XSERVER_LIBPCIACCESS
     xf86SetOperatingState(resVgaIo, pNv->pEnt->index, ResUnusedOpr);
     xf86SetOperatingState(resVgaMem, pNv->pEnt->index, ResDisableOpr);
+#endif
 
     /* Set pScrn->monitor */
     pScrn->monitor = pScrn->confScreen->monitor;
@@ -1452,8 +1448,6 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
 	return FALSE;
     }
     
-    xf86LoaderReqSymLists(vgahwSymbols, NULL);
-
     /*
      * Allocate a vgaHWRec
      */
@@ -1522,7 +1516,6 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
 	    return FALSE;
 	}
 	
-	xf86LoaderReqSymLists(fbdevHWSymbols, NULL);
 	if (!fbdevHWInit(pScrn, pNv->PciInfo, NULL)) {
 	    xf86FreeInt10(pNv->pInt);
 	    return FALSE;
@@ -1630,7 +1623,6 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
                        "enabled.\n");
             return FALSE;
         }
-        xf86LoaderReqSymLists(vbeModeSymbols, NULL);
         pNv->pVbe = VBEExtendedInit(NULL, pNv->pEnt->index,
                                     SET_BIOS_SCRATCH | RESTORE_BIOS_SCRATCH);
         if (!pNv->pVbe) return FALSE;
@@ -1705,6 +1697,7 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
     xf86DrvMsg(pScrn->scrnIndex, from, "MMIO registers at 0x%lX\n",
 	       (unsigned long)pNv->IOAddress);
      
+#ifndef XSERVER_LIBPCIACCESS
     if (xf86RegisterResources(pNv->pEnt->index, NULL, ResExclusive)) {
 	xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
 		"xf86RegisterResources() found resource conflicts\n");
@@ -1712,6 +1705,7 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
 	NVFreeRec(pScrn);
 	return FALSE;
     }
+#endif
 
     switch (pNv->Chipset & 0x0ff0) {
     case 0x0100:   /* GeForce 256 */
@@ -1962,8 +1956,6 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
 	return FALSE;
     }
 
-    xf86LoaderReqSymLists(fbSymbols, NULL);
-    
     /* Load XAA if needed */
     if (!pNv->NoAccel) {
 	if (!xf86LoadSubModule(pScrn, "xaa")) {
@@ -1971,7 +1963,6 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
 	    NVFreeRec(pScrn);
 	    return FALSE;
 	}
-	xf86LoaderReqSymLists(xaaSymbols, NULL);
     }
 
     /* Load ramdac if needed */
@@ -1981,7 +1972,6 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
 	    NVFreeRec(pScrn);
 	    return FALSE;
 	}
-	xf86LoaderReqSymLists(ramdacSymbols, NULL);
     }
 
     /* Load shadowfb if needed */
@@ -1991,7 +1981,6 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
 	    NVFreeRec(pScrn);
 	    return FALSE;
 	}
-	xf86LoaderReqSymLists(shadowSymbols, NULL);
     }
 
     pNv->CurrentLayout.bitsPerPixel = pScrn->bitsPerPixel;
